`timescale 1ns / 1ps

//******************************************************************************
//* Legnagyobb kzs oszt szmt modul.                                      *
//******************************************************************************
module lnko(
   input  wire       clk,           //rajel
   input  wire       rst,           //Reset jel
   input  wire       start,         //j szmts indtsa.
   input  wire [7:0] ai,            //'A' adatbemenet
   input  wire       lda,           //Az 'A' adat betlt jele
   input  wire [7:0] bi,            //'B' adatbemenet
   input  wire       ldb,           //A 'B' adat betlt jele
   output reg  [7:0] ao,            //Adatregiszterek
   output reg  [7:0] bo,
   output wire       ready,         //Szmts indthat jelzs
   output wire [2:0] fsm_state      //A vezrl aktulis llapota
);

//******************************************************************************
//* Az 'A' operandust s a rszeredmnyeket trol regiszter.                  *
//* A regiszter a port listban van deklarlva (ao).                           *
//******************************************************************************
wire [7:0] diff;     //A kivon ltal szmtott klnbsg.
wire       ena;      //Betltst engedlyez jel.
wire       sela;     //Betltend adat kivlaszts: ai (0) / klnbsg (1).

always @(posedge clk)
begin
   if (rst)
      ao <= 8'd0;
   else
      if (ena)
         ao <= (sela) ? diff : ai;   
end


//******************************************************************************
//* A 'B' operandust s a rszeredmnyeket trol regiszter.                   *
//* A regiszter a port listban van deklarlva (bo).                           *
//******************************************************************************
wire       enb;      //Betltst engedlyez jel.
wire       selb;     //Betltend adat kivlaszts: bi (0) / klnbsg (1).

always @(posedge clk)
begin
   if (rst)
      bo <= 8'd0;
   else
      if (enb)
         bo <= (selb) ? diff : bi;   
end


//******************************************************************************
//* A kivons operandusait felcserl multiplexerek s a kivon.               *
//******************************************************************************
wire       rsub;     //Az operandus csert vezrl jel (0: norml, 1: csere).

//A kivons operandusainak felcserlst vgz multiplexerek.
wire [7:0] sub_a_in = (rsub) ? bo : ao;
wire [7:0] sub_b_in = (rsub) ? ao : bo;

//A kivon modul (binris).
bin_sub_8bit subtractor(.a(sub_a_in), .b(sub_b_in), .diff(diff));

//A kivon modul (2 digites BCD).
//bcd_sub_8bit subtractor(.a(sub_a_in), .b(sub_b_in), .diff(diff));


//******************************************************************************
//* Kompartor.                                                                *
//******************************************************************************
//Az 'A' regiszter rtke nagyobb, mint a 'B' regiszter rtke.
wire agtb = (ao > bo);

//A 'B' regiszter rtke nagyobb, mint az 'A' regiszter rtke.
wire bgta = (bo > ao);


//******************************************************************************
//* Vezrl llapotgp.                                                        *
//******************************************************************************
localparam IDLE  = 3'd0;
localparam LOADA = 3'd1;
localparam LOADB = 3'd2;
localparam TEST  = 3'd3;
localparam SUB   = 3'd4;
localparam RSUB  = 3'd5;

//Letiltjuk az llapotkdols megvltoztatst, optimalizlst.
(* fsm_encoding = "user" *)
reg [2:0] state;

//Az llapotregiszter s a kvetkez llapot logika kzs always blokkban.
always @(posedge clk)
begin
   if (rst)
      state <= IDLE;
   else
      case (state)
         IDLE   : if (start)
                     state <= LOADA;
                  else
                     state <= IDLE;
         
         LOADA  : if (lda)
                     state <= LOADB;
                  else
                     state <= LOADA;
                     
         LOADB  : if (ldb)
                     state <= TEST;
                  else
                     state <= LOADB;
                     
         TEST   : if (agtb)
                     state <= SUB;
                  else
                     if (bgta)
                         state <= RSUB;
                     else
                         state <= IDLE;
                         
         SUB    : state <= TEST;
         
         RSUB   : state <= TEST;
         
         //rvnytelen llapotok.
         default: state <= IDLE;
      endcase
end


//Kimeneti logika.
assign ready = (state == IDLE);
assign ena   = (state == LOADA) | (state == SUB);
assign sela  = (state == SUB);
assign enb   = (state == LOADB) | (state == RSUB);
assign selb  = (state == RSUB);
assign rsub  = (state == RSUB);

//Kivezetjk az aktulis llapotot.
assign fsm_state = state;

endmodule
